<?php
namespace Ak\Db\Service;

use Ak\Db\Exception\ConfigurationException;
use Doctrine\DBAL\Configuration;
use Doctrine\DBAL\DBALException;
use Doctrine\DBAL\Driver\Connection;
use Doctrine\DBAL\DriverManager;
use Interop\Container\ContainerInterface;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Exception\ServiceNotFoundException;
use Zend\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Stvara nove instance konekcija na bazu.
 *
 * @package Ak\Db\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
final class ConnectionAbstractFactory implements AbstractFactoryInterface
{
    /** @var array */
    private $config;

    /**
     * Da li možemo stvoriti konekciju s datim imenom?
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        return (bool) preg_match('/^db(\.|$)/', $requestedName);
    }

    /**
     * Stvara konekciju.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return Connection
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        if (!is_array($options)) {
            $options = $this->getConfigForConnection($container, $requestedName);
        }

        $events = $container->get('db.events');

        try {
            return DriverManager::getConnection($options, new Configuration(), $events);
        } catch (DBALException $e) {
            throw new ServiceNotCreatedException('Ne mogu stvoriti konekciju.', 0, $e);
        }
    }

    /**
     * Vraća konfiguraciju za konekciju s datim imenom.
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @return array
     */
    private function getConfigForConnection(ContainerInterface $container, string $requestedName): array
    {
        $config = $this->getConfig($container);
        $name = $this->getConnectionName($container, $requestedName);

        if (!isset($config['connections'][$name])) {
            throw new ServiceNotFoundException(sprintf(
                "Konekcija s imenom '%s' ne postoji.", $name
            ));
        }

        $config = (array) $config['connections'][$name];

        if (($config['driver'] ?? '') === 'pdo_mysql' && !isset($config['charset'])) {
            $config['charset'] = 'utf8mb4';
        }

        return $config;
    }

    /**
     * Vraća relevantnu konfiguraciju.
     *
     * @param ContainerInterface $container
     * @return array
     */
    private function getConfig(ContainerInterface $container): array
    {
        if (null === $this->config) {
            $this->config = $container->get('config')['db'] ?? [];
        }

        return $this->config;
    }

    /**
     * Iz datog imena izvlači stvarno ime tražene konekcije.
     *
     * @param ContainerInterface $container
     * @param string $name
     * @return string
     */
    private function getConnectionName(ContainerInterface $container, string $name): string
    {
        if (!preg_match('/^db\.?([a-z0-9_]+)?$/', $name, $matches)) {
            throw new ServiceNotFoundException(sprintf(
                "Neispravno ime konekcije '%s', mora odgovarati regexu '%s'.",
                $name, '/^db\.?([a-z0-9_]+)?$/'
            ));
        }

        return explode('.', $name)[1] ?? $this->getDefaultConnectionName($container);
    }

    /**
     * Vraća zadano ime konekcije.
     *
     * @param ContainerInterface $container
     * @return string
     */
    private function getDefaultConnectionName(ContainerInterface $container): string
    {
        $config = $this->getConfig($container);

        if (!isset($config['default_connection']) || empty($config['default_connection'])) {
            throw new ConfigurationException('Nije postavljena zadana konekcija.');
        }

        return (string) $config['default_connection'];
    }
}