<?php
namespace AkTest\Db\Service;

use Ak\Db\ConfigProvider;
use Ak\Db\Service\ConnectionAbstractFactory;
use Ak\Test\AbstractTestCase;
use Doctrine\DBAL\DBALException;
use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Driver\PDOSqlite\Driver;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;

/**
 * Class ConnectionAbstractFactoryTest
 *
 * @package AkTest\Db\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
final class ConnectionAbstractFactoryTest extends AbstractTestCase
{
    public $configProviders = [
        ConfigProvider::class,
    ];

    public function testWillTryToCreateAnyValidName()
    {
        $container = $this->getContainer();
        $factory = new ConnectionAbstractFactory();

        $this->assertTrue($factory->canCreate($container, 'db'));
        $this->assertTrue($factory->canCreate($container, 'db.anything'));

        $this->assertFalse($factory->canCreate($container, 'anything_else'));
        $this->assertFalse($factory->canCreate($container, 'db_not_a_connection'));
    }

    /**
     * @expectedException \Zend\ServiceManager\Exception\ServiceNotFoundException
     */
    public function testWillThrowAnExceptionForInvalidConnectionName()
    {
        $container = $this->getContainer();
        $factory = new ConnectionAbstractFactory();

        $factory($container, 'invalid');
    }

    /**
     * @expectedException \Ak\Db\Exception\ConfigurationException
     */
    public function testWillThrowAnExceptionIfDefaultConnectionNotSet()
    {
        $container = $this->getContainer([
            'db' => [
                'default_connection' => null,
            ],
        ]);
        $factory = new ConnectionAbstractFactory();

        $factory($container, 'db');
    }

    /**
     * @expectedException \Zend\ServiceManager\Exception\ServiceNotFoundException
     */
    public function testWillThrowAnExceptionIfDefaultConnectionDoesntExist()
    {
        $container = $this->getContainer([
            'db' => [
                'default_connection' => 'doesnt_exist',
            ]
        ]);
        $factory = new ConnectionAbstractFactory();

        $factory($container, 'db');
    }

    public function testCanCreateADefaultConnection()
    {
        $container = $this->getContainer([
            'db' => [
                'default_connection' => 'default',

                'connections' => [
                    'default' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);
        $factory = new ConnectionAbstractFactory();

        /** @var \Doctrine\DBAL\Connection $connection */
        $connection = $factory($container, 'db');

        $this->assertInstanceOf(Connection::class, $connection);
        $this->assertInstanceOf(Driver::class, $connection->getDriver());
    }

    public function testCanCreateNamedConnection()
    {
        $container = $this->getContainer([
            'db' => [
                'connections' => [
                    'some_db' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);
        $factory = new ConnectionAbstractFactory();

        /** @var \Doctrine\DBAL\Connection $connection */
        $connection = $factory($container, 'db.some_db');

        $this->assertInstanceOf(Connection::class, $connection);
    }

    public function testWillCatchDBALExceptions()
    {
        $container = $this->getContainer([
            'db' => [
                'connections' => [
                    'some_db' => [

                    ],
                ],
            ],
        ]);
        $factory = new ConnectionAbstractFactory();

        try {
            $factory($container, 'db.some_db');
        } catch (\Throwable $e) {
            $this->assertInstanceOf(ServiceNotCreatedException::class, $e);
            $this->assertInstanceOf(DBALException::class, $e->getPrevious());
        }
    }

    public function testCorrectEventManagerIsInjected()
    {
        $container = $this->getContainer([
            'db' => [
                'connections' => [
                    'default' => [
                        'driver' => 'pdo_sqlite',
                        'memory' => true,
                    ],
                ],
            ],
        ]);
        $events = $container->get('db.events');
        /** @var Connection $connection */
        $connection = $container->get('db');

        $this->assertSame($events, $connection->getEventManager());
    }
}